/*
*       Copyright (c) 2011-2022 by esd electronics gmbh
*
*  This software is copyrighted by and is the sole property of
*  esd gmbh.  All rights, title, ownership, or other interests
*  in the software remain the property of esd gmbh. This
*  software may only be used in accordance with the corresponding
*  license agreement.  Any unauthorized use, duplication, transmission,
*  distribution, or disclosure of this software is expressly forbidden.
*
*  This Copyright notice may not be removed or modified without prior
*  written consent of esd gmbh.
*
*  esd gmbh, reserves the right to modify this software without notice.
*
*  esd electronics gmbh                   Tel. +49-511-37298-0
*  Vahrenwalder Strasse 207               Fax. +49-511-37298-68
*  30165 Hannover                         http://www.esd.eu
*  Germany                                sales@esd.eu
*
*************************************************************************
*
* Description:
* =============================================================================
* This file is used to configure the stack when using/building it from source.
* When the stack is used as binary it just shows how it was built.
* Used by Stack, HAL and application.
*
*/

#ifndef ESS_CONFIG_INCLUDED
#define ESS_CONFIG_INCLUDED


#ifdef ESS_BUILD_LIB /* Used by esd to build the library versions of the stack */
# define ESS_ESD_LIBRARY
#endif

#ifdef ESS_ESD_LIBRARY /* Define this when using the stack as library to have the same settings that were used when building them */
# define CFG_ESS_MAX_DEVICES                            4
# define CFG_ESS_MAX_MBX_LEN                            1486
# define CFG_ESS_OD_ALLOW_HANDLE_PDO_PARAMS             1
# define CFG_ESS_SERVE_ERR_LED                          1
# define CFG_ESS_SERVE_RUN_LED                          1
# define CFG_ESS_SUPPORT_FOE                            1
# define CFG_ESS_SUPPORT_EOE                            1
# define CFG_ESS_SUPPORT_VOE                            1
# define CFG_ESS_SUPPORT_SOE                            1
# define CFG_ESS_SUPPORT_AOE                            1
# define CFG_ESS_DC_CALLBACK                            1
# define CFG_ESS_SERVE_STATISTICS                       1
#endif



#ifndef CFG_HAL_EXTERNAL                                /* Set this to 1 on when HAL is in external library */
# define CFG_HAL_EXTERNAL                               0
#endif

#ifndef CFG_HAL_NEEDS_EVENTS                            /* Only used for some specific HALs which need to be informed about certain stack actions */
# define CFG_HAL_NEEDS_EVENTS                           0
#endif

#ifndef CFG_HAL_USE_SYNC0_IRQ                           /* HAL is able to detect ESC SYNC0 IRQ */
# define CFG_HAL_USE_SYNC0_IRQ                          0
#endif

#ifndef CFG_HAL_USE_SYNC1_IRQ                           /* HAL is able to detect ESC SYNC1 IRQ */
# define CFG_HAL_USE_SYNC1_IRQ                          0
#endif

#ifndef CFG_BIG_ENDIAN                                  /* Set this to 1 on big endian systems */
# define CFG_BIG_ENDIAN                                 0
#endif

#ifndef CFG_HAVE_STD_INT_TYPES                          /* Set this to 1 when e.g. <inttypes.h> was included previously, else ess.h will define 'int8_t', etc. itself */
# define CFG_HAVE_STD_INT_TYPES                         0
#endif

#ifndef CFG_ESC_HAVE_POINTER                            /* Set this to 1 when ESC memory is accessed by pointer - acquired by essHALMapESC(). Set to 0 when stack shall use HAL's API to read/write ESC memory - e.g. with SPI access */
# define CFG_ESC_HAVE_POINTER                           1
#endif

#ifndef CFG_ESC_HAVE_READ_FUNC                          /* Set this to 1 when variable length ESC memory reads have a special function although otherwise pointer is used */
# define CFG_ESC_HAVE_READ_FUNC                         0
#endif

#ifndef CFG_ESC_HAVE_WRITE_FUNC                         /* Set this to 1 when variable length ESC memory writes have a special function although otherwise pointer is used */
# define CFG_ESC_HAVE_WRITE_FUNC                        0
#endif

#ifndef CFG_ESS_MAX_DEVICES                             /* Max devIdx supported, e.g. in essOpen() */
# define CFG_ESS_MAX_DEVICES                            1
#endif

#ifndef CFG_ESS_MAX_SM_COUNT                            /* Number of sync manager on the ESC (Maximum is 16) */
# define CFG_ESS_MAX_SM_COUNT                           8
#endif

#ifndef CFG_ESS_MAX_MBX_LEN                             /* Max 1486 (Min depends on supported mbx protocol) */
# define CFG_ESS_MAX_MBX_LEN                            1024
#endif

#ifndef CFG_ESS_MIN_SM_ALLOC                            /* Min size to allocate for SM buffer copy (Might be increased to avoid realloc calls) */
# define CFG_ESS_MIN_SM_ALLOC                           0x0100
#endif

#ifndef CFG_ESS_MAX_EEPROM_WAIT                         /* Max wait for cleared busy bit (in ms) */
# define CFG_ESS_MAX_EEPROM_WAIT                        50
#endif

#ifndef CFG_ESS_EEPROM_ACK_WAIT                         /* Wait before cmd retry because of missing ACK (in ms, max 10 retries) */
# define CFG_ESS_EEPROM_ACK_WAIT                        5
#endif

#ifndef CFG_ESS_EEPROM_EMULATION                        /* When enabled, ESC EEPROM is emulated. (Stack triggers callbacks to let the application access the data) */
# define CFG_ESS_EEPROM_EMULATION                       0
#endif

#ifndef CFG_ESS_SERVE_ERR_LED                           /* Set this to 1 to let the stack trigger the ERR LED (No effect when HAL has not implemented LED access) */
# define CFG_ESS_SERVE_ERR_LED                          0
#endif

#ifndef CFG_ESS_SERVE_RUN_LED                           /* Set this to 1 to let the stack trigger the RUN LED (No effect when HAL has not implemented LED access) */
# define CFG_ESS_SERVE_RUN_LED                          0
#endif

#ifndef CFG_ESS_SERVE_CUSTOM_LED                        /* Set this to 0 to reduce code size if CFG_ESS_SERVE_ERR_LED and CFG_ESS_SERVE_RUN_LED are also 0 / HAL does not implement LEDs */
# define CFG_ESS_SERVE_CUSTOM_LED                       1
#endif

#ifndef CFG_ESS_COMBINED_RUNERR_LED                     /* Set this to 1 when ERR and RUN LED are combined in a single LED, stack will then set RUN LED to off while ERR LED is not off */
# define CFG_ESS_COMBINED_RUNERR_LED                    0
#endif

#ifndef CFG_ESS_SUPPORT_COE                             /* Determines whether the COE functions are enabled */
# define CFG_ESS_SUPPORT_COE                            1
#endif

#ifndef CFG_ESS_COE_SUPPORT_COMPLETE_ACCESS             /* Not implemented, must be 0 */
# define CFG_ESS_COE_SUPPORT_COMPLETE_ACCESS            0
#endif

#ifndef CFG_ESS_COE_SDO_SEGMENT_TIMEOUT                 /* Timeout for SDO transfer: when last segment receipt is longer ago than this, transfer is aborted */
# define CFG_ESS_COE_SDO_SEGMENT_TIMEOUT                3000 /* ms */
#endif

#ifndef CFG_ESS_COE_SUPPORT_SDO_INFO_SERVICE            /* Support SDO info service (Master can read the CoE dictionary from the slave) */
# define CFG_ESS_COE_SUPPORT_SDO_INFO_SERVICE           1
#endif

#ifndef CFG_ESS_COE_OBJECT_LIST_INCREMENT               /* Number of new items allocated in dictionary's object list when there's no space for next object left (each is a pointer) */
# define CFG_ESS_COE_OBJECT_LIST_INCREMENT              4
#endif

#ifndef CFG_ESS_COE_ENTRY_LIST_INCREMENT                /* Number of new items allocated in object's entry list when there's no space for next entry left (each is a pointer) */
# define CFG_ESS_COE_ENTRY_LIST_INCREMENT               4
#endif

#ifndef CFG_ESS_COE_MAPPED_LIST_INCREMENT               /* For each SM a list of entries mapped to it exists. This is the number of new items allocated when there's no space for the next item left (each is a ESS_PDO_MAP_INFO struct) */
# define CFG_ESS_COE_MAPPED_LIST_INCREMENT              4
#endif

#ifndef CFG_ESS_COE_MAX_ITEMS_PER_PDO_OBJECT            /* Max value for "entryCount" in essODUpdatePDOConfiguration() */
# define CFG_ESS_COE_MAX_ITEMS_PER_PDO_OBJECT           255 /* Max. 255 */
#endif

#ifndef CFG_ESS_COE_MAX_ITEMS_PER_PDOASSIGN_OBJECT      /* Max value for "pdoCount" in essODUpdatePDOAssignment() */
# define CFG_ESS_COE_MAX_ITEMS_PER_PDOASSIGN_OBJECT     255 /* Max. 255 */
#endif

#ifndef CFG_ESS_OD_ALLOW_HANDLE_SM_TYPE                 /* Determines whether ESS_OD_FLAGS_HANDLE_SM_TYPES can be used. (Automatic creation of Object 0x1c00) */
# define CFG_ESS_OD_ALLOW_HANDLE_SM_TYPE                1
#endif

#ifndef CFG_ESS_OD_ALLOW_HANDLE_PDO_PARAMS              /* Determines whether the essODPDOParam functions are enabled. */
# define CFG_ESS_OD_ALLOW_HANDLE_PDO_PARAMS             0
#endif

#ifndef CFG_ESS_SUPPORT_FOE                             /* Determines whether the FOE functions are enabled */
# define CFG_ESS_SUPPORT_FOE                            0
#endif

#ifndef CFG_ESS_FOE_TRANSFER_TIMEOUT                    /* Timeout for FoE transfers: when last data packet is longer ago than this, transfer is aborted */
# define CFG_ESS_FOE_TRANSFER_TIMEOUT                   5000 /* ms */
#endif

#ifndef CFG_ESS_SUPPORT_EOE                             /* Determines whether the EOE functions are enabled */
# define CFG_ESS_SUPPORT_EOE                            0
#endif

#ifndef CFG_ESS_SUPPORT_VOE                             /* Determines whether the VOE callback is enabled */
# define CFG_ESS_SUPPORT_VOE                            0
#endif

#ifndef CFG_ESS_SUPPORT_SOE                             /* Determines whether the SOE callback is enabled */
# define CFG_ESS_SUPPORT_SOE                            0
#endif

#ifndef CFG_ESS_SUPPORT_AOE                             /* Determines whether the AOE callback is enabled */
# define CFG_ESS_SUPPORT_AOE                            0
#endif

#ifndef CFG_ESS_SM_CALLBACK                             /* Set this to 1 when SM interrupts might be caught by the application (cbSMInterrupt in ESS_CONFIGURATION) */
# define CFG_ESS_SM_CALLBACK                            0
#endif

#ifndef CFG_ESS_DC_CALLBACK                             /* Set this to 1 when cbDCEvent in ESS_CONFIGURATION shall be used */
# define CFG_ESS_DC_CALLBACK                            0
#endif

#ifndef CFG_ESS_COPY_CALLBACKS                          /* Set this to 0 to save some memory (sizeof ESS_CALLBACKS), 1 for minor performance gain */
# define CFG_ESS_COPY_CALLBACKS                         1
#endif

#ifndef CFG_ESS_SERVE_STATISTICS                        /* Set this to 1 to serve statistics given with ESS_CONFIGURATION in essOpen() (stats pointer is mandatory then) */
# define CFG_ESS_SERVE_STATISTICS                       0
#endif

#ifndef CFG_ESS_SMCOPYINUSERBUFFER                      /* For future use. Stack internally allocates memory to store SM buffer copies - by this the app gives bufferpointers that are used instead (ESS_SMBUFFERCOPY* in config) */
# define CFG_ESS_SMCOPYINUSERBUFFER                     0
#endif

#ifndef CFG_ESS_COE_ALLOWTOMAPNULLENTRIES               /* For internal use only. By this CoE entries with data == NULL can be mapped into a PDO (so data of these entries within PDOs will be never updated/unsued) */
# define CFG_ESS_COE_ALLOWTOMAPNULLENTRIES              0
#endif

#ifndef CFG_ESS_DISABLE_DEPRECATED                      /* Some functions exist only to maintain backward compatibility, set this to 1 to remove these (possibly reducing code size) */
# define CFG_ESS_DISABLE_DEPRECATED                     0
#endif

#ifndef CFG_ESS_EXPLICIT_DEVICE_ID                      /* Set to 1 to support the explicit device ID mechanism described in the specification ETG.1020 */
# define CFG_ESS_EXPLICIT_DEVICE_ID                     1
#endif

/* Other macros: */
/*****************/


/* Version information: */
#define ESS_LEVEL                                       1
#define ESS_REVI                                        3
#define ESS_BUILD                                       16

#ifndef ESS_BUILD_SPECIAL                               /* e.g. "_BETA1" - will be appended to version string */
# define ESS_BUILD_SPECIAL                              ""
#endif

#define ESS_ABI_VERSION                                 7

#define ESS_VERSION                                     STR_STRINGI(ESS_LEVEL.ESS_REVI.ESS_BUILD) ESS_VERSION_SUFFIX
#ifdef _DEBUG
# define ESS_VERSION_SUFFIX                             "D" ESS_BUILD_SPECIAL
#else
# define ESS_VERSION_SUFFIX                             ESS_BUILD_SPECIAL
#endif

#define STR_STRINGI(s)                                  STRINGI(s)
#define STRINGI(s)                                      #s


#ifndef DLL_EXPORT
# ifdef WINDOWS
#  define DLL_EXPORT                                    __declspec(dllexport)
# elif __GNUC__ >= 4
#   define DLL_EXPORT                                   __attribute__((visibility("default")))
# else
#   define DLL_EXPORT
# endif
#endif

#ifndef DLL_IMPORT
# ifdef WINDOWS
#  define DLL_IMPORT                                    __declspec(dllimport)
# else
#  define DLL_IMPORT
# endif
#endif

#ifndef ESS_EXPORT
# ifdef ESS_BUILD_LIB
#  define ESS_EXPORT                                    DLL_EXPORT
# else
#  define ESS_EXPORT                                    DLL_IMPORT
# endif
#endif

#ifndef HAL_EXPORT
# ifdef HAL_BUILD_LIB
#  define HAL_EXPORT                                    DLL_EXPORT
# elif CFG_HAL_EXTERNAL
#  define HAL_EXPORT                                    DLL_IMPORT
# else
#  define HAL_EXPORT
# endif
#endif

#ifndef ESS_CALLTYPE
# define ESS_CALLTYPE
#endif

#ifndef HAL_CALLTYPE
# define HAL_CALLTYPE
#endif


#endif /* ESS_CONFIG_INCLUDED */
