/*
*       Copyright (c) 2011-2022 by esd electronics gmbh
*
*  This software is copyrighted by and is the sole property of
*  esd gmbh.  All rights, title, ownership, or other interests
*  in the software remain the property of esd gmbh. This
*  software may only be used in accordance with the corresponding
*  license agreement.  Any unauthorized use, duplication, transmission,
*  distribution, or disclosure of this software is expressly forbidden.
*
*  This Copyright notice may not be removed or modified without prior
*  written consent of esd gmbh.
*
*  esd gmbh, reserves the right to modify this software without notice.
*
*  esd electronics gmbh                   Tel. +49-511-37298-0
*  Vahrenwalder Strasse 207               Fax. +49-511-37298-68
*  30165 Hannover                         http://www.esd.eu
*  Germany                                sales@esd.eu
*
*************************************************************************
*
* Description:
* =============================================================================
* This file handles platform specific includes, macros, etc.
* Used by Stack and application.
*
*/

#ifndef ESS_SYSTEM_INCLUDED_
#define ESS_SYSTEM_INCLUDED_


#if defined(WINDOWS) || defined(ESS_PLATFORM_WINDOWS)
#define _CRT_SECURE_NO_WARNINGS
# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <windows.h>
# include <time.h>
# define _ESS_GETTHREADID() ((void*)(uintptr_t)GetCurrentThreadId())
# define ESS_PLATFORM_WINDOWS


#elif defined(LINUX) || defined(linux) || defined(ESS_PLATFORM_LINUX)
# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <time.h>
# include <unistd.h>
# include <inttypes.h>
# include <string.h>
# include <sched.h>
# ifdef _GNU_SOURCE
#  include <sys/syscall.h>
#  define _ESS_GETTHREADID()  ((void*)(uintptr_t)syscall(SYS_gettid))
# endif
# define CFG_HAVE_STD_INT_TYPES 1
# define ESS_PLATFORM_LINUX


#elif defined(ESS_PLATFORM_AM335X)
# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <time.h>
# include <inttypes.h>
# include <string.h>
# include <xdc/runtime/Timestamp.h>
# define CFG_HAVE_STD_INT_TYPES 1
  extern void MyUARTPutString(const char* s);

  static void ConsolePrintF(const char* fmt, ...) __attribute__((format(printf, 1, 2)));
  static void ConsolePrintF(const char* fmt, ...)
  {
      va_list _vl;
      char _tmp[256];

      va_start(_vl, (fmt));
      vsnprintf(_tmp, sizeof(_tmp) - 1, (fmt), _vl);
      _tmp[sizeof(_tmp) - 1] = '\0';
      va_end(_vl);

      MyUARTPutString(_tmp);
  }
# define ESS_PRINTF ConsolePrintF


#elif defined(__QNX__) || defined(ESS_PLATFORM_QNX)
# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <inttypes.h>
# include <string.h>
# include <sys/neutrino.h>
# include <gulliver.h>
# define CFG_HAVE_STD_INT_TYPES 1
# define ESS_PLATFORM_QNX

#define _POSIX_C_SOURCE 200809


#elif defined(ESS_PLATFORM_RIN32M3)
# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <inttypes.h>
# include <string.h>
# define CFG_HAVE_STD_INT_TYPES 1

#elif defined(ESS_PLATFORM_XMC4800)
# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <inttypes.h>
# include <string.h>
# define CFG_HAVE_STD_INT_TYPES 1

#elif defined(ESS_PLATFORM_VXWORKS)
  /*
   *  Below is a workaround for VxWorks 7.x which will fail to include <stdlib.h>
   *  if __RTP__ is undefined ??.
   */
# if !defined(__RTP__)
#  define __RTP__
#  include <stdlib.h>
#  undef __RTP__
#else   
#  include <stdlib.h>
#endif  
# include <stdio.h>
# include <string.h>
# define CFG_HAVE_STD_INT_TYPES 0
  
#elif defined(ESS_PLATFORM_DUMMY)
# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <inttypes.h>
# include <string.h>
# define CFG_HAVE_STD_INT_TYPES 1


#else
# error "Unknown/undefined platform. Please define it or add a new one in essSystem.h (essPrivate.h might need changes then, too)"
#endif

  /*
   * Define a macro to mark a variable as unused to suppress compiler warnings
   */
#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 5)
# define _ESS_UNUSED_ __attribute__ ((__unused__))
#else
# define _ESS_UNUSED_
#endif

#ifndef ESS_PRINTF                      /* Used by essPrivate.h e.g. for stack's debug macros. But application can also use it. */
# define ESS_PRINTF                     printf
#endif

#ifndef ESS_PRINTFFLUSH                 /* used by ess's default debug macros, might be used to add a fflush() */
# define ESS_PRINTFFLUSH()              do { } while (0)
#endif

#endif /* ESS_SYSTEM_INCLUDED_ */
