/*
*       Copyright (c) 2011-2020 by esd electronics gmbh
*
*  This software is copyrighted by and is the sole property of
*  esd gmbh.  All rights, title, ownership, or other interests
*  in the software remain the property of esd gmbh. This
*  software may only be used in accordance with the corresponding
*  license agreement.  Any unauthorized use, duplication, transmission,
*  distribution, or disclosure of this software is expressly forbidden.
*
*  This Copyright notice may not be removed or modified without prior
*  written consent of esd gmbh.
*
*  esd gmbh, reserves the right to modify this software without notice.
*
*  esd electronics gmbh                   Tel. +49-511-37298-0
*  Vahrenwalder Strasse 207               Fax. +49-511-37298-68
*  30165 Hannover                         http://www.esd.eu
*  Germany                                sales@esd.eu
*
*************************************************************************
*
* Description:
* =============================================================================
* Defines basic ess types. Used by Stack, HAL and application.
*
*/


#ifndef ESS_TYPES_INCLUDED
#define ESS_TYPES_INCLUDED

#include "essConfig.h"

#ifndef NULL
# define NULL ((void*)0)
#endif

#if (!CFG_HAVE_STD_INT_TYPES)
    /* It's recommended to have these types defined by standard includes. essOpen() performs sizeof checks to make sure these are OK. */
    typedef unsigned char uint8_t;
    typedef signed char int8_t;

    typedef unsigned short uint16_t;
    typedef signed short int16_t;

# ifdef __INT32_TYPE__
    typedef __INT32_TYPE__ int32_t;
# elif defined(_MSC_VER)
    typedef __int32 int32_t;
# else
    typedef signed int int32_t;
# endif
# ifdef __UINT32_TYPE__
    typedef __UINT32_TYPE__ uint32_t;
# elif defined(_MSC_VER)
    typedef unsigned __int32 uint32_t;
# else
    typedef unsigned int uint32_t;
# endif

# ifdef __UINT64_TYPE__
    typedef __UINT64_TYPE__ uint64_t;
# elif defined(_MSC_VER)
    typedef unsigned __int64 uint64_t;
# elif !defined(__STRICT_ANSI__)
    typedef unsigned long long uint64_t;
# endif

# ifdef __INT64_TYPE__
    typedef __INT64_TYPE__ int64_t;
# elif defined(_MSC_VER)
    typedef __int64 int64_t;
# elif !defined(__STRICT_ANSI__)
    typedef signed long long int64_t;
# endif

# ifndef _UINTPTR_T_DEFINED
#  ifdef __UINTPTR_TYPE__
    typedef __UINTPTR_TYPE__ uintptr_t;
#  else
    typedef unsigned long uintptr_t;
#  endif
#  define _UINTPTR_T_DEFINED
# endif
#endif /* !CFG_HAVE_STD_INT_TYPES */


#include "ecatDefs.h"

typedef void* ESS_HANDLE;

/*lint -strong(Ab, ESS_BOOL) */
typedef int8_t ESS_BOOL;
#define ESS_FALSE                                       ((ESS_BOOL)0)
#define ESS_TRUE                                        ((ESS_BOOL)1)

typedef uint16_t ESS_RESULT;
#define ESS_RESULT_SUCCESS                              ((ESS_RESULT)   0)
#define ESS_RESULT_UNKNOWN                              ((ESS_RESULT)  -1)
#define ESS_RESULT_INVALID_INDEX                        ((ESS_RESULT)  -2)
#define ESS_RESULT_NOT_READY                            ((ESS_RESULT)  -3)
#define ESS_RESULT_ALREADY_ACTIVE                       ((ESS_RESULT)  -4)
#define ESS_RESULT_CANT_OPEN                            ((ESS_RESULT)  -5)
#define ESS_RESULT_TRY_AGAIN                            ((ESS_RESULT)  -6)
#define ESS_RESULT_MISSING_CALLBACK                     ((ESS_RESULT)  -7)
#define ESS_RESULT_INVALID_HANDLE                       ((ESS_RESULT)  -8)
#define ESS_RESULT_INVALID_LIST                         ((ESS_RESULT)  -9)
#define ESS_RESULT_OBJ_NOT_FOUND                        ((ESS_RESULT) -10)
#define ESS_RESULT_ENTRY_NOT_FOUND                      ((ESS_RESULT) -11)
#define ESS_RESULT_NO_DICT                              ((ESS_RESULT) -12)
#define ESS_RESULT_NO_MEM                               ((ESS_RESULT) -13)
#define ESS_RESULT_OBJ_EXISTS                           ((ESS_RESULT) -14)
#define ESS_RESULT_ENTRY_EXISTS                         ((ESS_RESULT) -15)
#define ESS_RESULT_WRONG_ECAT_STATE                     ((ESS_RESULT) -16)
#define ESS_RESULT_INVALID_CONFIG                       ((ESS_RESULT) -17)
#define ESS_RESULT_INVALID_OD_FLAGS                     ((ESS_RESULT) -18)
#define ESS_RESULT_INVALID_SUBINDEX                     ((ESS_RESULT) -19)
#define ESS_RESULT_INVALID_SM                           ((ESS_RESULT) -20)
#define ESS_RESULT_INVALID_ARG                          ((ESS_RESULT) -21)
#define ESS_RESULT_NOT_IMPLEMENTED                      ((ESS_RESULT) -22)
#define ESS_RESULT_INVALID_SIZE                         ((ESS_RESULT) -23)
#define ESS_RESULT_BUSY                                 ((ESS_RESULT) -24)
#define ESS_RESULT_INVALID_PDI_CONTROL                  ((ESS_RESULT) -25)
#define ESS_RESULT_ABI_INCOMPATIBLE                     ((ESS_RESULT) -26)
#define ESS_RESULT_INVALID_TIMER_CONFIG                 ((ESS_RESULT) -27)
#define ESS_RESULT_MISSING_ACK                          ((ESS_RESULT) -28)
#define ESS_RESULT_INVALID_OBJECT                       ((ESS_RESULT) -29)
#define ESS_RESULT_INSUFFICIENT_BUFFER                  ((ESS_RESULT) -30)
#define ESS_RESULT_INVALID_PDI_CONFIGURATION            ((ESS_RESULT) -31)
#define ESS_RESULT_ERROR_EEPROM_CONTROL                 ((ESS_RESULT) -32)
/* Don't forget to add new entries to essFormatResult(), too. */

typedef uint32_t ESS_TIMESTAMP;
#define ESS_TIMESTAMP_PASSED(ts, now)                   ((int32_t)((ts) - (now)) <= 0)

typedef union {
  uint64_t tick64;
  struct {
    uint32_t low;
    uint32_t high;
  } tick;
} ESS_SYSTIME;

typedef uint8_t ESS_DEVICE_INDEX;

typedef uint8_t ESC_LED_TYPE;
#define ESC_LED_TYPE_NUMBER_OF_LEDS                     ((ESC_LED_TYPE)0x04)
#define ESC_LED_TYPE_ERR                                ((ESC_LED_TYPE)0x00)
#define ESC_LED_TYPE_RUN                                ((ESC_LED_TYPE)0x01)
#define ESC_LED_TYPE_CUSTOM1                            ((ESC_LED_TYPE)0x02)
#define ESC_LED_TYPE_CUSTOM2                            ((ESC_LED_TYPE)0x03)
#define ESC_LED_TYPE_TO_STRING(t)                       ((t) == ESC_LED_TYPE_ERR) ? "ERR" : ((t) == ESC_LED_TYPE_RUN) ? "RUN" : ((t) == ESC_LED_TYPE_CUSTOM1) ? "CUSTOM1" : ((t) == ESC_LED_TYPE_CUSTOM2) ? "CUSTOM2" : "UNKNOWN!?"

typedef uint8_t ESC_LED_STATE;
#define ESC_LED_STATE_OFF                               ((ESC_LED_STATE)0x00)
#define ESC_LED_STATE_FLASH_1                           ((ESC_LED_STATE)0x01)
#define ESC_LED_STATE_FLASH_2                           ((ESC_LED_STATE)0x02)
#define ESC_LED_STATE_FLASH_3                           ((ESC_LED_STATE)0x03)
#define ESC_LED_STATE_FLASH_4                           ((ESC_LED_STATE)0x04)
#define ESC_LED_STATE_BLINK                             ((ESC_LED_STATE)0x0d)
#define ESC_LED_STATE_FLICKER                           ((ESC_LED_STATE)0x0e)
#define ESC_LED_STATE_ON                                ((ESC_LED_STATE)0x0f)
#define ESC_LED_STATE_TO_STRING(lState)                 ((lState) == ESC_LED_STATE_OFF) ? "OFF" : ((lState) == ESC_LED_STATE_FLASH_1) ? "1 x FLASH" : ((lState) == ESC_LED_STATE_FLASH_2) ? "2 x FLASH" : ((lState) == ESC_LED_STATE_FLASH_3) ? "3 x FLASH" : ((lState) == ESC_LED_STATE_FLASH_4) ? "4 x FLASH" : ((lState) == ESC_LED_STATE_BLINK) ? "BLINK" : ((lState) == ESC_LED_STATE_FLICKER) ? "FLICKER" : ((lState) == ESC_LED_STATE_ON) ? "ON" : "UNKNOWN!?"

typedef uint16_t ESS_OD_FLAGS;
#define ESS_OD_FLAGS_NONE                               ((ESS_OD_FLAGS)0)
#define ESS_OD_FLAGS_HANDLE_SM_TYPES                    ((ESS_OD_FLAGS)(1 << 0))
#define ESS_OD_SDO_INFO_MODE_ETG1020                    ((ESS_OD_FLAGS)(1 << 1))

typedef uint16_t ESS_OD_OBJECT_FLAGS;
#define ESS_OD_OBJECT_FLAGS_NONE                        ((ESS_OD_OBJECT_FLAGS)0)
#define ESS_OD_OBJECT_FLAGS_UNIQUE_ENTRY_NAMES          ((ESS_OD_OBJECT_FLAGS)(1 << 0))

typedef uint8_t ESS_OD_ENTRY_FLAGS;
#define ESS_OD_ENTRY_FLAGS_NONE                         ((ESS_OD_ENTRY_FLAGS)0)
#define ESS_OD_ENTRY_FLAGS_UPLOAD_TRIMMED               ((ESS_OD_ENTRY_FLAGS)(1 << 0))
#define ESS_OD_ENTRY_FLAGS_USE_DATA_AS_MAXSUBIDX        ((ESS_OD_ENTRY_FLAGS)(1 << 1))
#define ESS_OD_ENTRY_FLAGS_NOSDOINFO                    ((ESS_OD_ENTRY_FLAGS)(1 << 2))
#define ESS_OD_ENTRY_FLAGS_ALLOWPARTIALDOWNLOAD         ((ESS_OD_ENTRY_FLAGS)(1 << 3))
#define ESS_OD_ENTRY_FLAGS_DYNAMIC_SIZE                 ((ESS_OD_ENTRY_FLAGS)(1 << 4))

typedef uint16_t ESS_OD_ENTRY_INDEX;
#define ESS_OD_ENTRY_INDEX_ALL                          ((ESS_OD_ENTRY_INDEX)-1)

typedef int8_t ESS_PDO_CFG_FLAGS;
#define ESS_PDO_CFG_FLAGS_NONE                          ((ESS_PDO_CFG_FLAGS)0)
#define ESS_PDO_CFG_FLAGS_WRITABLE_IN_PREOP             ((ESS_PDO_CFG_FLAGS)(1 << 0))
#define ESS_PDO_CFG_FLAGS_BLOCK_COPY                    ((ESS_PDO_CFG_FLAGS)(1 << 1))

typedef uint16_t ESS_OD_PDOPARAM_FLAGS;
#define ESS_OD_PDOPARAM_FLAGS_NONE                      ((ESS_OD_PDOPARAM_FLAGS)0)
#define ESS_OD_PDOPARAM_FLAGS_EXCLPDOS_EXISTS           ((ESS_OD_PDOPARAM_FLAGS)(1 << 0))
#define ESS_OD_PDOPARAM_FLAGS_STATE_EXISTS              ((ESS_OD_PDOPARAM_FLAGS)(1 << 1))
#define ESS_OD_PDOPARAM_FLAGS_CONTROL_EXISTS            ((ESS_OD_PDOPARAM_FLAGS)(1 << 2))
#define ESS_OD_PDOPARAM_FLAGS_TOGGLE_EXISTS             ((ESS_OD_PDOPARAM_FLAGS)(1 << 3))

typedef uint32_t ESS_ISYN_FLAGS;
#define ESS_ISYN_FLAGS_NONE                             ((ESS_ISYN_FLAGS)0)
#define ESS_ISYN_FLAGS_SM0_IMMEDIATELY                  ((ESS_ISYN_FLAGS)(1 << 0))
#define ESS_ISYN_FLAGS_SM1_IMMEDIATELY                  ((ESS_ISYN_FLAGS)(1 << 1))
#define ESS_ISYN_FLAGS_SM2_IMMEDIATELY                  ((ESS_ISYN_FLAGS)(1 << 2))
#define ESS_ISYN_FLAGS_SM3_IMMEDIATELY                  ((ESS_ISYN_FLAGS)(1 << 3))
#define ESS_ISYN_FLAGS_SM4_IMMEDIATELY                  ((ESS_ISYN_FLAGS)(1 << 4))
#define ESS_ISYN_FLAGS_SM5_IMMEDIATELY                  ((ESS_ISYN_FLAGS)(1 << 5))
#define ESS_ISYN_FLAGS_SM6_IMMEDIATELY                  ((ESS_ISYN_FLAGS)(1 << 6))
#define ESS_ISYN_FLAGS_SM7_IMMEDIATELY                  ((ESS_ISYN_FLAGS)(1 << 7))
#define ESS_ISYN_FLAGS_SM8_IMMEDIATELY                  ((ESS_ISYN_FLAGS)(1 << 8))
#define ESS_ISYN_FLAGS_SM9_IMMEDIATELY                  ((ESS_ISYN_FLAGS)(1 << 9))
#define ESS_ISYN_FLAGS_SM10_IMMEDIATELY                 ((ESS_ISYN_FLAGS)(1 << 10))
#define ESS_ISYN_FLAGS_SM11_IMMEDIATELY                 ((ESS_ISYN_FLAGS)(1 << 11))
#define ESS_ISYN_FLAGS_SM12_IMMEDIATELY                 ((ESS_ISYN_FLAGS)(1 << 12))
#define ESS_ISYN_FLAGS_SM13_IMMEDIATELY                 ((ESS_ISYN_FLAGS)(1 << 13))
#define ESS_ISYN_FLAGS_SM14_IMMEDIATELY                 ((ESS_ISYN_FLAGS)(1 << 14))
#define ESS_ISYN_FLAGS_SM15_IMMEDIATELY                 ((ESS_ISYN_FLAGS)(1 << 15))

typedef uint16_t ESS_OD_ENTRY_CALLBACK;
#define ESS_OD_ENTRY_CALLBACK_REQUESTED_UPLOAD          ((ESS_OD_ENTRY_CALLBACK)0x0001)
#define ESS_OD_ENTRY_CALLBACK_REQUESTED_DOWNLOAD        ((ESS_OD_ENTRY_CALLBACK)0x0002)
#define ESS_OD_ENTRY_CALLBACK_STARTING_UPLOAD           ((ESS_OD_ENTRY_CALLBACK)0x0003)
#define ESS_OD_ENTRY_CALLBACK_STARTING_DOWNLOAD         ((ESS_OD_ENTRY_CALLBACK)0x0004)
#define ESS_OD_ENTRY_CALLBACK_COMPLETED_UPLOAD          ((ESS_OD_ENTRY_CALLBACK)0x0005)
#define ESS_OD_ENTRY_CALLBACK_COMPLETED_DOWNLOAD        ((ESS_OD_ENTRY_CALLBACK)0x0006)
#define ESS_OD_ENTRY_CALLBACK_ABORTED_UPLOAD            ((ESS_OD_ENTRY_CALLBACK)0x0007)
#define ESS_OD_ENTRY_CALLBACK_ABORTED_DOWNLOAD          ((ESS_OD_ENTRY_CALLBACK)0x0008)
#define ESS_OD_ENTRY_CALLBACK_GET_UPLOAD_SIZE           ((ESS_OD_ENTRY_CALLBACK)0x0009)
#define ESS_OD_ENTRY_CALLBACK_IS_UPLOAD_EVENT(e)        (((e) == ESS_OD_ENTRY_CALLBACK_REQUESTED_UPLOAD) || ((e) == ESS_OD_ENTRY_CALLBACK_STARTING_UPLOAD) || ((e) == ESS_OD_ENTRY_CALLBACK_COMPLETED_UPLOAD) || ((e) == ESS_OD_ENTRY_CALLBACK_ABORTED_UPLOAD))
#define ESS_OD_ENTRY_CALLBACK_IS_DOWNLOAD_EVENT(e)      (((e) == ESS_OD_ENTRY_CALLBACK_REQUESTED_DOWNLOAD) || ((e) == ESS_OD_ENTRY_CALLBACK_STARTING_DOWNLOAD) || ((e) == ESS_OD_ENTRY_CALLBACK_COMPLETED_DOWNLOAD) || ((e) == ESS_OD_ENTRY_CALLBACK_ABORTED_DOWNLOAD))
#define ESS_OD_ENTRY_CALLBACK_TO_STRING(cbT)            (((cbT) == ESS_OD_ENTRY_CALLBACK_REQUESTED_UPLOAD) ? \
                                                        "REQUESTED_UPLOAD" : ((cbT) == ESS_OD_ENTRY_CALLBACK_REQUESTED_DOWNLOAD) ? \
                                                        "REQUESTED_DOWNLOAD" : ((cbT) == ESS_OD_ENTRY_CALLBACK_STARTING_UPLOAD) ? \
                                                        "STARTING_UPLOAD" : ((cbT) == ESS_OD_ENTRY_CALLBACK_STARTING_DOWNLOAD) ? \
                                                        "STARTING_DOWNLOAD" : ((cbT) == ESS_OD_ENTRY_CALLBACK_COMPLETED_UPLOAD) ? \
                                                        "COMPLETED_UPLOAD" : ((cbT) == ESS_OD_ENTRY_CALLBACK_COMPLETED_DOWNLOAD) ? \
                                                        "COMPLETED_DOWNLOAD" : ((cbT) == ESS_OD_ENTRY_CALLBACK_ABORTED_UPLOAD) ? \
                                                        "ABORTED_UPLOAD" : ((cbT) == ESS_OD_ENTRY_CALLBACK_ABORTED_DOWNLOAD) ? \
                                                        "ABORTED_DOWNLOAD" : ((cbT) == ESS_OD_ENTRY_CALLBACK_GET_UPLOAD_SIZE) ? \
                                                        "GET_UPLOAD_SIZE" : "Unknown")

typedef uint32_t ESS_CONFIG_FLAGS;
#define ESS_CONFIG_FLAGS_NONE                           ((ESS_CONFIG_FLAGS)0)
/* When used HAL offers an ISR, this flags determines if it's used */
#define ESS_CONFIG_FLAGS_USE_ISR                        ((ESS_CONFIG_FLAGS)(1 << 0))
/* Must be set when application supports bootstrap mode */
#define ESS_CONFIG_FLAGS_APPSUPP_BOOTSTRAP              ((ESS_CONFIG_FLAGS)(1 << 1))

typedef uint8_t ESS_EVENT;
/* Called before first byte of MBoxOut is read */
#define ESS_EVENT_READMBOXOUTSTART                      ((ESS_EVENT)0x01)
/* Called after last byte of MBoxOut was read */
#define ESS_EVENT_READMBOXOUTEND                        ((ESS_EVENT)0x02)
/* Called after first byte of MBoxIn was written */
#define ESS_EVENT_WRITEMBOXINSTART                      ((ESS_EVENT)0x03)
/* Called after last byte of MBoxIn was written */
#define ESS_EVENT_WRITEMBOXINEND                        ((ESS_EVENT)0x04)
/* Called before Inputs SM buffer is written */
#define ESS_EVENT_NEEDSMADDRINPUTS                      ((ESS_EVENT)0x05)
/* Called after Inputs SM buffer was written */
#define ESS_EVENT_WROTEINPUTS                           ((ESS_EVENT)0x06)
/* Called before Outputs SM buffer is read */
#define ESS_EVENT_NEEDSMADDROUTPUTS                     ((ESS_EVENT)0x07)
/* Called after Outputs SM buffer was read */
#define ESS_EVENT_READOUTPUTS                           ((ESS_EVENT)0x08)
/* Called after Reg. 0x0120 was read */
#define ESS_EVENT_READALCONTROL                         ((ESS_EVENT)0x09)
/* Called after Reg. 0x0130 was written */
#define ESS_EVENT_WROTEALSTATUS                         ((ESS_EVENT)0x0a)
/* Called after Reg. 0x0502 was written */
#define ESS_EVENT_WROTEEEPROMCTRL                       ((ESS_EVENT)0x0b)
/* Called after Reg. 0x0806 (+ 8 * smIdx) was read */
#define ESS_EVENT_READSMACTIVATE                        ((ESS_EVENT)0x0c)
/* Called after Reg. 0x0807 (+ 8 * smIdx) was written */
#define ESS_EVENT_WROTEPDICONTROL                       ((ESS_EVENT)0x0d)
/* Called after SMChanged interrupt was handled */
#define ESS_EVENT_SMUPDATED                             ((ESS_EVENT)0x10)
/* Called after SM was enabled */
#define ESS_EVENT_SMENABLED                             ((ESS_EVENT)0x11)
/* Called after SM was disabled */
#define ESS_EVENT_SMDISABLED                            ((ESS_EVENT)0x12)
/* Called to clear an AL event */
#define ESS_EVENT_CLEARALEVENT                          ((ESS_EVENT)0x13)


typedef uint8_t ESS_DC_EVENT;
/* SYNC0 interrupt event occurred */
#define ESS_DC_EVENT_SYNC0                              ((ESS_DC_EVENT)0x01)
/* SYNC1 interrupt event occurred */
#define ESS_DC_EVENT_SYNC1                              ((ESS_DC_EVENT)0x02)
/* LATCH0/LATCH1 interrupt event occurred */
#define ESS_DC_EVENT_LATCH                              ((ESS_DC_EVENT)0x03)

/* If ess does not know about an Ioctl it is forwarded to HAL. All ESS_IOCTLs with Bit31 set will never
   be used by esd and are always just forwarded to ess HAL - for customer specific HAL/IOCTLs. */
typedef uint32_t ESS_IOCTL;
#define ESS_IOCTL_GET_HAL_BUILDINFOSTR                  10
#define ESS_IOCTL_GET_DEV_DRIVERINFOSTR                 11
#define ESS_IOCTL_GET_DEV_PRODNAMESTR                   20
#define ESS_IOCTL_GET_DEV_ORDERNOSTR                    21
#define ESS_IOCTL_GET_DEV_SERIALNOSTR                   22
#define ESS_IOCTL_GET_DEV_HWVERSION32                   23
#define ESS_IOCTL_GET_DEV_HWFEATURES32                  24
#define ESS_IOCTL_GET_DEV_SERIALNO32                    25
#define ESS_IOCTL_DEV_RESET                             30
#define ESS_IOCTL_GET_DEV_TEMP16                        31 /* in 16th of deg C ([0.625..4095.875], 0x0000/0xffff used to indicate errors) */
#define ESS_IOCTL_DEV_RAWREAD                           32 /* IOCTL's arg is pointer to ESS_IOCTL_RAW_DATA */
#define ESS_IOCTL_DEV_RAWWRITE                          33
#define ESS_IOCTL_DEV_GPIO                              34 /* IOCTL's arg is pointer to ESS_IOCTL_GPIO */
#define ESS_IOCTL_DEV_CPUBOOT                           35
#define ESS_IOCTL_DEV_CPUCOMMAND                        36
#define ESS_IOCTL_TIESC_FLAGS                           37 /* Flags for TI PRUSS based ESC */

#define ESS_TIESC_PDI_WD_TRIGGER_CYCLIC                 (1 << 0)

typedef struct _ESS_IOCTL_RAW_DATA {
    void* data;
    uint32_t length;
    uint32_t target;
    uint32_t offset;
} ESS_IOCTL_RAW_DATA;

typedef struct _ESS_IOCTL_GPIO_DATA {
    uint8_t  bankselect;
    uint8_t  reserved[3];
    uint32_t input;               /* 0=low 1=high     */
    uint32_t direction;           /* 0=input 1=output */
    uint32_t direction_mask;      /* 0=ignore 1=set   */
    uint32_t output;              /* 0=low 1=high     */
    uint32_t output_mask;         /* 0=ignore 1=set   */
} ESS_IOCTL_GPIO_DATA;

typedef struct _ESS_EVENT_DATA {
    uint8_t devIdx;
    ESS_EVENT event;
    uint16_t arg1;
    uint16_t arg2;
    uint16_t arg3;
    uint16_t arg4;
    uint16_t result;
} ESS_EVENT_DATA;

typedef struct _ESS_OD_OBJECT_INFOS {
    const char* name;
    COE_DATATYPE dataType;
    COE_CODE code;
} ESS_OD_OBJECT_INFOS;

typedef struct _ESS_OD_ENTRY_INFOS {
    const char* name;
    const uint8_t* minValue;
    const uint8_t* maxValue;
    const uint8_t* defaultValue;
    uint32_t unitType;
    COE_DATATYPE dataType;
} ESS_OD_ENTRY_INFOS;

typedef struct _ESS_CBDATA_COE_READWRITE {
    ESS_HANDLE hDev;
    uint8_t* destSrc;
    uint32_t offset;
    COE_ABORTCODE abortCode;
    uint32_t len;
    uint16_t objIdx;
    uint8_t entryIdx;
    ESS_BOOL isRead;
    ESS_BOOL isLastPart;
} ESS_CBDATA_COE_READWRITE;

typedef struct _ESS_MBX_PACKET {
    /* valid len of data[] */
    uint16_t length;
    uint16_t address;
    /* Use MBX_HEADER_GET_CHANNEL() etc. */
    uint8_t channelAndPrio;
    /* Use MBX_HEADER_GET_COUNTER() etc. */
    uint8_t typeAndCounter;
    uint8_t data[CFG_ESS_MAX_MBX_LEN - MBX_HEADERLEN];
} ESS_MBX_PACKET;

typedef struct _ESS_CBDATA_CYCLIC {
    ESS_HANDLE hDev;
    /* In milliseconds */
    ESS_TIMESTAMP essTime;
} ESS_CBDATA_CYCLIC;

typedef struct _ESS_CBDATA_COE_EVENT {
    ESS_HANDLE hDev;
    COE_ABORTCODE abortCode;
    uint16_t objIdx;
    ESS_OD_ENTRY_CALLBACK cbType;
    uint8_t entryIdx;
} ESS_CBDATA_COE_EVENT;

typedef struct _ESS_COE_EMERGENCY {
    uint16_t errCode;
    uint8_t errReg;
    uint8_t data[5];
} ESS_COE_EMERGENCY;

    
#define ESS_PDO_IDX_PADDING     0x00        /* Index of padding object          */
#define ESS_PDO_SUB_PADDING     0x00        /* Sub index of padding object      */
#define ESS_PDO_SUB_EXTEND      0xFF        /* Sub index for objects > 31 bytes */

#define ESS_MAP_ENTRY(idx, sub, size)  {(idx), (sub), (size)}
#define ESS_MAP_EXTEND(size)  {ESS_PDO_IDX_PADDING, ESS_PDO_SUB_EXTEND, (size)}
#define ESS_MAP_DUMMY(size)  {ESS_PDO_IDX_PADDING, ESS_PDO_SUB_PADDING, (size)}

typedef struct _ESS_PDO_ENTRY {
    uint16_t objIdx;
    uint8_t subIdx;
    uint8_t len;
} ESS_PDO_ENTRY;

typedef struct _ESS_CBDATA_STATE_REQUEST {
    ESS_HANDLE hDev;
    /* lo byte is current state, hi byte is requested state (= newState) */
    ESC_TRANSITION transition;
    REG_VAL_ALSTATUSCODE statusCode;
    ESC_STATE newState;
    ESS_BOOL errAck;
    ESS_BOOL tryAgain;
    ESS_BOOL devIdReq;
    uint16_t devId;
} ESS_CBDATA_STATE_REQUEST;

/* If set outputs should be set to safe state. */
#define ESS_SM_EVENT_FLAG_SAFE_OUTPUTS      (1U << 31)

typedef struct _ESS_CBDATA_SM_EVENT {
    ESS_HANDLE hDev;
    /* For each input SM where an interrupt was signaled a bit is set here */
    uint32_t sm;
} ESS_CBDATA_SM_EVENT;

typedef struct _ESS_CBDATA_INOUTPUTS_ACTIVATE {
    ESS_HANDLE hDev;
    /* ESS_FALSE: outputs, else inputs */
    ESS_BOOL inputs;
    /* ESS_FALSE: stopped, else started */
    ESS_BOOL started;
} ESS_CBDATA_INOUTPUTS_ACTIVATE;

typedef struct _ESS_CBDATA_SM {
    ESS_HANDLE hDev;
    uint16_t smPhysAddr;   /* Last value read from ESC_REG_SMPHYSADDR(smIdx) */
    uint16_t smLen;        /* Last value read from ESC_REG_SMLEN(smIdx)      */
    uint8_t  smContrByte;  /* Last value read from ESC_REG_SMCONTROL(smIdx)  */
    ESS_SM   smIdx;        /* SM index 0 .. CFG_ESS_MAX_SM_COUNT             */
    uint8_t  smActivate;   /* Last value read from ESC_REG_SMACTIVATE(smIdx) */
    SM_TYPE  smType;       /* Derived from smContrByte                       */
} ESS_CBDATA_SM;

/*
 * Note: Each FoE related data structure indicates upload or download.
 *       Upload   :  Data transfer from Slave -> Master aka FoE_Read
 *       Download :  Data transfer from Master -> Slave aka FoE_Write
 */

typedef struct _ESS_CBDATA_FOE_OPEN {
    ESS_HANDLE hDev;
    /* Trailing NUL is ensured */
    const char* fileName;
    /* For custom usage by app (e.g. to store file handle to access/close file in other callbacks) */
    void* appHandle;
    uint32_t password;
    /* Application can set this to deny the FoE transfer */
    FOE_ERRORCODE errCode;
    /* can be used in conjunction with errCode */
    const char* errorText;
    /* Length of fileName */
    uint16_t fileNameLen;
    /* Net size of the mailbox */
    uint16_t maxDataLen;
    /* Set to ESS_TRUE/ESS_FALSE for FoE Upload/Download */
    ESS_BOOL isUpload;
} ESS_CBDATA_FOE_OPEN;

typedef struct _ESS_CBDATA_FOE_CLOSE {
    ESS_HANDLE hDev;
    /* For custom usage by app (e.g. to store file handle to access/close file in other callbacks) */
    const void* appHandle;
    /* Set to ESS_TRUE/ESS_FALSE for FoE Upload/Download */
    ESS_BOOL isUpload;
    /* true when a transfer timed out, etc. */
    ESS_BOOL unexpected;
} ESS_CBDATA_FOE_CLOSE;

typedef struct _ESS_CBDATA_FOE_DATA {
    ESS_HANDLE hDev;
    /* For custom usage by app (e.g. to store file handle to access/close file in other callbacks) */
    const void* appHandle;
    /* Might be set by app (in conjunction with errCode) */
    const char* errorText;
    /* Upload/Download : Address to copy data to/from */
    uint8_t* destSrc;
    /* Offset to read/write data initialized with 0 and adapted with each call */
    uint32_t offset;
    /* Application can set this to abort/stall the FoE transfer */
    FOE_ERRORCODE errCode;
    /* Number of download or upload bytes in destSrc (less than maxDataLen means EOT) */
    uint16_t len;
    /* Net size of the mailbox */
    uint16_t maxDataLen;
    /* Set to ESS_TRUE/ESS_FALSE for FoE Upload/Download */
    ESS_BOOL isUpload;
} ESS_CBDATA_FOE_DATA;

typedef struct _ESS_CBDATA_EOE_SETIPPARAM {
    ESS_HANDLE hDev;
    /* Pointer to 6 bytes, or NULL if wasn't included in the request */
    const uint8_t* macAddress;
    /* Pointer to 4 bytes (little endian), or NULL if wasn't included in the request */
    const uint8_t* ipAddress;
    /* Pointer to 4 bytes (little endian), or NULL if wasn't included in the request */
    const uint8_t* subnetMask;
    /* Pointer to 4 bytes (little endian), or NULL if wasn't included in the request */
    const uint8_t* defaultGateway;
    /* Pointer to 4 bytes (little endian), or NULL if wasn't included in the request */
    const uint8_t* dnsServerIP;
    /* Pointer to zero terminated string, or NULL if wasn't included in the request */
    const char* dnsName;
    /* Set by app */
    EOE_RESULTCODE result;
} ESS_CBDATA_EOE_SETIPPARAM;

typedef struct _ESS_CBDATA_EOE_SETADDRFILTER {
    ESS_HANDLE hDev;
    /* Pointer to macAddrCount*6 bytes (little endian) that represent the MAC addresses */
    const uint8_t* macAddresses;
    /* Pointer to macAddrFilterCount*6 bytes (little endian) that represent the MAC address filters */
    const uint8_t* macAddrFilter;
    /* Set by app */
    EOE_RESULTCODE result;
    /* No. of MAC addresses at macAddresses member */
    uint8_t macAddrCount;
    /* No. of filters at macAddrFilter member */
    uint8_t macAddrFilterCount;
    /* Whether to filter broadcast messages */
    ESS_BOOL filterBroadcasts;
} ESS_CBDATA_EOE_SETADDRFILTER;

typedef struct _ESS_CBDATA_EOE_FRAME {
    ESS_HANDLE hDev;
    uint8_t* data;
    uint32_t reserved1;
    uint16_t dataLen;
    uint8_t frameNo;
    ESS_BOOL reserved2;
} ESS_CBDATA_EOE_FRAME;

typedef struct _ESS_CBDATA_VOE {
    ESS_HANDLE hDev;
    ESS_MBX_PACKET* mb;
    /* Max value for mb->length in reply */
    uint16_t mbMaxLen;
    uint16_t result;
    ESS_BOOL fragment;
} ESS_CBDATA_VOE;

typedef struct _ESS_CBDATA_SOE {
    ESS_HANDLE hDev;
    ESS_MBX_PACKET* mb;
    /* Max value for mb->length in reply */
    uint16_t mbMaxLen;
    uint16_t result;
    ESS_BOOL fragment;
} ESS_CBDATA_SOE;

typedef struct _ESS_CBDATA_AOE {
    ESS_HANDLE hDev;
    ESS_MBX_PACKET* mb;
    /* Max value for mb->length in reply */
    uint16_t mbMaxLen;
    uint16_t result;
    ESS_BOOL fragment;
} ESS_CBDATA_AOE;

typedef struct _ESS_CBDATA_EEPROM_EMULATION {
    ESS_HANDLE hDev;
    /* isRead: data destination for read operation, else data source of write operation  */
    uint8_t* destSrc;
    /* Offset of EEPROM data to read/write in byte */
    uint32_t offset;
    /* len of data to read/write in byte */
    uint32_t len;
    /* App shall write 0 on success, 1 on error */
    uint8_t result;
    /* ESS_TRUE: data from EEPROM shall be read and stored at destSrc, else data at destSrc shall be written */
    ESS_BOOL isRead;
} ESS_CBDATA_EEPROM_EMULATION;


typedef struct _ESS_CBDATA_DC {
    ESS_HANDLE hDev;
    ESS_DC_EVENT evtType;
} ESS_CBDATA_DC;


typedef void (ESS_HAL_CALLBACK) (void*);
typedef void (ESS_CB_CYCLIC) (ESS_CBDATA_CYCLIC*);
typedef void (ESS_CB_STATE_REQUEST) (ESS_CBDATA_STATE_REQUEST*);
typedef void (ESS_CB_INOUTPUTS_ACTIVATE) (ESS_CBDATA_INOUTPUTS_ACTIVATE*);
typedef ESS_BOOL (ESS_CB_SYNCMANAGER) (ESS_CBDATA_SM*);
typedef void (ESS_CB_OUTPUTS_UPDATED) (ESS_CBDATA_SM_EVENT*);
typedef void (ESS_CB_INPUTS_UPDATED) (ESS_CBDATA_SM_EVENT*);
typedef void (ESS_CB_COE_READWRITE) (ESS_CBDATA_COE_READWRITE*);
typedef void (ESS_CB_COE_EVENT) (ESS_CBDATA_COE_EVENT*);
typedef void (ESS_CB_FOE_OPEN) (ESS_CBDATA_FOE_OPEN*);
typedef void (ESS_CB_FOE_CLOSE) (ESS_CBDATA_FOE_CLOSE*);
typedef void (ESS_CB_FOE_DATA) (ESS_CBDATA_FOE_DATA*);
typedef void (ESS_CB_EOE_SETIPPARAM) (ESS_CBDATA_EOE_SETIPPARAM*);
typedef void (ESS_CB_EOE_SETADDRFILTER) (ESS_CBDATA_EOE_SETADDRFILTER*);
typedef void (ESS_CB_EOE_FRAME) (ESS_CBDATA_EOE_FRAME*);
typedef void (ESS_CB_VOE) (ESS_CBDATA_VOE*);
typedef void (ESS_CB_EEPROM_EMULATION) (ESS_CBDATA_EEPROM_EMULATION*);
typedef void (ESS_CB_DC) (ESS_CBDATA_DC*);
typedef void (ESS_CB_SOE) (ESS_CBDATA_SOE*);
typedef void (ESS_CB_AOE) (ESS_CBDATA_AOE*);




typedef struct _ESS_STATISTICS {
    /* Incremented when SM bit in AL Event (0x0220) is set, i.e. also works for polling etc. */
    uint32_t smInterrupts[16];
    /* Incremented each time the stack's MainLoop() is called */
    uint32_t totalCalls;
    /* Incremented each time the stack's MainLoop() is called by HAL's cyclic handler */
    uint32_t cyclicCalls;
    /* Incremented for each EoE frame sent to EtherCAT (when last chunk was written to mailbox) */
    uint32_t eoeSent;
    /* Incremented for each EoE frame received from EtherCAT */
    uint32_t eoeReceived;
} ESS_STATISTICS;

typedef struct _ESS_SM_CONFIGURATION {
    uint16_t minSize;
    uint16_t defSize;
    uint16_t maxSize;
    uint16_t startAddr;
    uint8_t contrByte;
} ESS_SM_CONFIGURATION;

/* Some callbacks are mandatory, see documentation. */
typedef struct _ESS_CALLBACKS {
    ESS_CB_CYCLIC* cbCyclic;
    ESS_CB_STATE_REQUEST* cbStateRequest;
    ESS_CB_SYNCMANAGER* cbSMInterrupt;
    ESS_CB_INOUTPUTS_ACTIVATE* cbInOutputsActivate;
    ESS_CB_OUTPUTS_UPDATED* cbOutputsUpdated;
    ESS_CB_INPUTS_UPDATED* cbInputsUpdated;
    ESS_CB_COE_READWRITE* cbCoEReadWrite;
    ESS_CB_COE_EVENT* cbCoEEvent;
    ESS_CB_FOE_OPEN* cbFoEOpen;
    ESS_CB_FOE_CLOSE* cbFoEClose;
    ESS_CB_FOE_DATA* cbFoEData;
    ESS_CB_EOE_SETIPPARAM* cbEoESetIPParam;
    ESS_CB_EOE_SETADDRFILTER* cbEoESetAddrFilter;
    ESS_CB_EOE_FRAME* cbEoEFrame;
    ESS_CB_VOE* cbVoE;
    ESS_CB_EEPROM_EMULATION* cbEEPROMEmulation;
    ESS_CB_DC* cbDCEvent;
    ESS_CB_SOE* cbSoE;
    ESS_CB_AOE* cbAoE;
} ESS_CALLBACKS;

typedef struct _ESS_SM_USERBUFFER {
    void* data;
    uint16_t capacity;
} ESS_SM_USERBUFFER;

typedef struct _ESS_CONFIGURATION {
    /* Use ESS_ABI from essConfig.h */
    uint32_t essABIVersion;
    ESS_CONFIG_FLAGS flags;
    uint32_t timerInterval;
    void* tag;
    const ESS_CALLBACKS* cb;
    const ESS_SM_CONFIGURATION* smConfigs;
    uint8_t smConfigCount;
    uint8_t smConfigCountBootstrap;
    /* reserved values must be 0 */
    uint8_t reserved[2];
    /* Mandatory with CFG_ESS_SERVE_STATISTICS, else ignored. */
    ESS_STATISTICS* stats;
    /* For future use (Must point to CFG_ESS_MAX_SM_COUNT items then) */
    const ESS_SM_USERBUFFER* smBufferCopies;
} ESS_CONFIGURATION;

#endif  /* ESS_TYPES_INCLUDED */
